local mod = CreateFrame('Frame')

local factionChamps
if GetLocale() == 'enUS' then
	factionChamps = {
		-- alliance npcs
		['Tyrius Duskblade'] 		= 'DEATHKNIGHT',
		['Kavina Grovesong'] 		= 'DRUID',
		['Melador Valestrider'] 	= 'DRUID',
		['Alyssia Moonstalker'] 	= 'HUNTER',
		['Noozle Whizzlestick']		= 'MAGE',
		['Baelnor Lightbearer'] 	= 'PALADIN',
		['Velanaa']					= 'PALADIN', -- I know I can use shorthand for these indices but I like consistency!
		['Anthar Forgemender'] 		= 'PRIEST',
		['Brienna Nightfell'] 		= 'PRIEST',
		['Irieth Shadowstep'] 		= 'ROGUE',
		['Saamul'] 					= 'SHAMAN',
		['Shaabad'] 				= 'SHAMAN',
		['Serissa Grimdabbler']		= 'WARLOCK',
		['Shocuul'] 				= 'WARRIOR',
		
		-- horde npcs
		['Gorgrim Shadowcleave'] 	= 'DEATHKNIGHT',
		['Birana Stormhoof'] 		= 'DRUID',
		['Erin Misthoof'] 			= 'DRUID',
		["Ruj'kah"] 				= 'HUNTER',
		['Ginselle Blightslinger'] 	= 'MAGE',
		['Liandra Suncaller'] 		= 'PALADIN',
		['Malithas Brightblade'] 	= 'PALADIN',
		['Caiphus the Stern'] 		= 'PRIEST',
		['Vivienne Blackwhisper'] 	= 'PRIEST',
		["Maz'dinah"] 				= 'ROGUE',
		['Thrakgar'] 				= 'SHAMAN',
		['Broln Stouthorn'] 		= 'SHAMAN',
		['Harkzog'] 				= 'WARLOCK',
		['Narrhok Steelbreaker']	= 'WARRIOR'
	}
elseif GetLocale() == 'frFR' then -- frFR translation by Mang (http://wow.curse.com/user/Profile.aspx?UserID=272295)
	factionChamps = {
		-- alliance npcs
		['Tyrius Lamebrune'] 		= 'DEATHKNIGHT', -- Tyrius Duskblade
		['Kavina Chantebosquet'] 	= 'DRUID', -- Kavina Grovesong
		['Melador Arpenteval'] 		= 'DRUID', -- Melador Valestrider
		['Alyssia Traquelune'] 		= 'HUNTER', -- Alyssia Moonstalker
		['Touillert Vizitige'] 		= 'MAGE', -- Noozle Whizzlestick
		['Baelnor Portelumire'] 	= 'PALADIN', -- Baelnor Lightbearer
		['Velanaa'] 				= 'PALADIN', -- Velanaa
		['Anthar Soigneforge'] 		= 'PRIEST', -- Anthar Forgemender
		['Brienna Tombenuit'] 		= 'PRIEST', -- Brienna Nightfell
		["Irieth Pas-de-l'ombre"] 	= 'ROGUE', -- Irieth Shadowstep
		['Saamul']				 	= 'SHAMAN', -- Saamul
		['Shaabad'] 				= 'SHAMAN', -- Shaabad
		['Serissa Funbricole'] 	= 'WARLOCK', -- Serissa Grimdabbler
		['Shocuul'] 				= 'WARRIOR', -- Shocuul

		-- horde npcs
		['Gorgrim Fend-les-ombres'] = 'DEATHKNIGHT', -- Gorgrim Shadowcleave
		['Birana Sabot-tempte']	= 'DRUID', -- Birana Stormhoof
		['Erin Sabot-de-brume'] 	= 'DRUID', -- Erin Misthoof
		["Ruj'kah"] 				= 'HUNTER', -- Ruj'kah
		['Ginselle Jettechancre'] 	= 'MAGE', -- Ginselle Blightslinger
		['Liandra Mande-soleil'] 	= 'PALADIN', -- Liandra Suncaller
		['Malithas Brillelame'] 	= 'PALADIN', -- Malithas Brightblade
		['Caiphus le Svre'] 		= 'PRIEST', -- Caiphus the Stern
		['Vivienne Murmenoir'] 		= 'PRIEST', -- Vivienne Blackwhisper
		["Maz'dinah"] 				= 'ROGUE', -- Maz'dinah
		['Thrakgar'] 				= 'SHAMAN', -- Thrakgar
		['Broln Corne-rude'] 		= 'SHAMAN', -- Broln Stouthorn
		['Harkzog'] 				= 'WARLOCK', -- Harkzog
		['Narrhok Brise-acier'] 	= 'WARRIOR' -- Narrhok Steelbreaker
	}
end

local colors
mod:RegisterEvent('PLAYER_LOGIN')
mod:SetScript('OnEvent', function()
	colors = CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS
end)

local frames = {}
local FrameIsNameplate = function(frame)
	-- Nameplates are unnamed children of the WorldFrame. Anything with a name isn't a nameplate... Ironically.
	if frame:GetName() then return end
	
	local region = frame:GetRegions()
	return not frames[frame] and region and region:GetObjectType() == 'Texture' and region:GetTexture() == 'Interface\\TargetingFrame\\UI-TargetingFrame-Flash'
end

local CheckNameplateColor = function(frame)
	local _, _, _, _, _, _, name = frame:GetParent():GetRegions()
	if not name.SetFont then return end
	
	if factionChamps[name:GetText()] then
		local c = colors[factionChamps[name:GetText()]]
		frame:SetStatusBarColor(c.r, c.g, c.b)
	end
end

local numChildren = -1
mod:SetScript('OnUpdate', function()
	if WorldFrame:GetNumChildren() ~= numChildren then
		numChildren = WorldFrame:GetNumChildren()
		
		-- Find those pesky nameplates.
		for i = 1, select('#', WorldFrame:GetChildren()) do
			local frame = select(i, WorldFrame:GetChildren())
		
			if FrameIsNameplate(frame) then
				frames[frame] = true
				
				local health, cast = frame:GetChildren()
				health:HookScript('OnShow', CheckNameplateColor)
				CheckNameplateColor(health)
			end
		end
	end
end)